/*! For license information please see 967.2aaf8ca7c37a92081b62.js.LICENSE.txt */
"use strict";(self.webpackChunk_jupyterlab_git=self.webpackChunk_jupyterlab_git||[]).push([[967],{8967:(t,i,e)=>{e.r(i),e.d(i,{filesize:()=>E,partial:()=>N});const o="array",n="bit",r="bits",l="byte",a="bytes",s="",b="exponent",p="function",u="iec",c="Invalid number",f="Invalid rounding method",d="jedec",g="object",h=".",m="round",y="s",B="kbit",M="kB",w=" ",k="string",j="0",x={symbol:{iec:{bits:["bit","Kibit","Mibit","Gibit","Tibit","Pibit","Eibit","Zibit","Yibit"],bytes:["B","KiB","MiB","GiB","TiB","PiB","EiB","ZiB","YiB"]},jedec:{bits:["bit","Kbit","Mbit","Gbit","Tbit","Pbit","Ebit","Zbit","Ybit"],bytes:["B","KB","MB","GB","TB","PB","EB","ZB","YB"]}},fullform:{iec:["","kibi","mebi","gibi","tebi","pebi","exbi","zebi","yobi"],jedec:["","kilo","mega","giga","tera","peta","exa","zetta","yotta"]}};function E(t,{bits:i=!1,pad:e=!1,base:E=-1,round:N=2,locale:T=s,localeOptions:P={},separator:G=s,spacer:K=w,symbols:S={},standard:Y=s,output:Z=k,fullform:_=!1,fullforms:v=[],exponent:z=-1,roundingMethod:I=m,precision:O=0}={}){let $=z,C=Number(t),L=[],q=0,A=s;-1===E&&0===Y.length?(E=10,Y=d):-1===E&&Y.length>0?E=(Y=Y===u?u:d)===u?2:10:Y=10==(E=2===E?2:10)||Y===d?d:u;const D=10===E?1e3:1024,F=!0===_,H=C<0,J=Math[I];if("bigint"!=typeof t&&isNaN(t))throw new TypeError(c);if(typeof J!==p)throw new TypeError(f);if(H&&(C=-C),(-1===$||isNaN($))&&($=Math.floor(Math.log(C)/Math.log(D)),$<0&&($=0)),$>8&&(O>0&&(O+=8-$),$=8),Z===b)return $;if(0===C)L[0]=0,A=L[1]=x.symbol[Y][i?r:a][$];else{q=C/(2===E?Math.pow(2,10*$):Math.pow(1e3,$)),i&&(q*=8,q>=D&&$<8&&(q/=D,$++));const t=Math.pow(10,$>0?N:0);L[0]=J(q*t)/t,L[0]===D&&$<8&&-1===z&&(L[0]=1,$++),A=L[1]=10===E&&1===$?i?B:M:x.symbol[Y][i?r:a][$]}if(H&&(L[0]=-L[0]),O>0&&(L[0]=L[0].toPrecision(O)),L[1]=S[L[1]]||L[1],!0===T?L[0]=L[0].toLocaleString():T.length>0?L[0]=L[0].toLocaleString(T,P):G.length>0&&(L[0]=L[0].toString().replace(h,G)),e&&!1===Number.isInteger(L[0])&&N>0){const t=G||h,i=L[0].toString().split(t),e=i[1]||s,o=e.length,n=N-o;L[0]=`${i[0]}${t}${e.padEnd(o+n,j)}`}return F&&(L[1]=v[$]?v[$]:x.fullform[Y][$]+(i?n:l)+(1===L[0]?s:y)),Z===o?L:Z===g?{value:L[0],symbol:L[1],exponent:$,unit:A}:L.join(K)}function N({bits:t=!1,pad:i=!1,base:e=-1,round:o=2,locale:n=s,localeOptions:r={},separator:l=s,spacer:a=w,symbols:b={},standard:p=s,output:u=k,fullform:c=!1,fullforms:f=[],exponent:d=-1,roundingMethod:g=m,precision:h=0}={}){return s=>E(s,{bits:t,pad:i,base:e,round:o,locale:n,localeOptions:r,separator:l,spacer:a,symbols:b,standard:p,output:u,fullform:c,fullforms:f,exponent:d,roundingMethod:g,precision:h})}}}]);